@@ -123,7 +123,7 @@ typedef struct batinfo_field {
 
 #define UEVENT_ENUM_FIELDS(_)						\
   _(state,   BATTERY,  STATUS,  "Discharging","Charging","Full","Unknown" ) \
-  _(type,    BOTH,     TYPE,    "Mains",       "Battery"              )
+  _(type,    BOTH,     TYPE,    "Mains",       "Battery",  "USB" )
 
 #define CHGST_DISCHARGING 0 /* Reflects order in _(state,...) above     */
 #define CHGST_CHARGING    1 /* Also, much code assumes exactly          */
@@ -132,8 +132,9 @@ typedef struct batinfo_field {
 #define CHGST_ERROR       8 /* Except that this one is an extra bit.    */
 
 #define TYPE_MAINS        0 /* Reflects order in _(type,...) above        */
-#define TYPE_BATTERY      1 /* Also, much code assumes exactly these two  */
-#define TYPE_BOTH       100 /* Except this is a magic invalid value.      */
+#define TYPE_BATTERY      1 /* We want exactly one of MAINS & BATTERY     */
+#define TYPE_USB          2 /* These get ignored.                         */
+#define TYPE_BOTH       100 /* This is a magic invalid value.             */
 
 #define SEPARATE_QUANTITY_FIELDS(_)		\
   /* See commit ec6f5f0be800bc5f2a27046833dba04e0c67ffac for
@@ -243,10 +244,15 @@ static int parse_value(const fileinfo *cfile, const batinfo_field *field) {
 
   if (!field->enumarray[0]) {
 
+    // Some kernels present a negative number for POWER_SUPPLY_CURRENT_NOW
+    // #1109342, #1109344
+    const char *usevalue = batlinevalue;
+    if (*usevalue == '-') usevalue++;
+
     char *ep;
-    *field->valuep= strtoull(batlinevalue,&ep,10);
+    *field->valuep= strtoull(usevalue,&ep,10);
     if (*ep)
-      batfailf("value number syntax incorrect");
+      return batfailf("value number syntax incorrect");
 
   } else {
 	
@@ -255,7 +261,7 @@ static int parse_value(const fileinfo *cfile, const batinfo_field *field) {
 	 *enumsearch && strcmp(*enumsearch,batlinevalue);
 	 (*field->valuep)++, enumsearch++);
     if (!*enumsearch)
-      batfailf("unknown enum value");
+      return batfailf("unknown enum value");
 
   }
   return 0;
@@ -396,6 +402,9 @@ ALL_ACCUMULATE_FIELDS(Q_ZERO)
     r= readbattery();
     tidybattery();
 
+    if (this_type >= TYPE_USB)
+      continue;
+
     if (r) {
     bad:
       charging_mask |= (1u << CHGST_ERROR);
